@@ -82,6 +82,50 @@ trim_string(char **pstr)
   str[i] = '\0';
 }
 
+/* Split PROPERTY and store each individual value in PROPS.
+   Allocates from POOL. */
+static void
+split_props(apr_array_header_t **props,
+            const char *property,
+            apr_pool_t *pool)
+{
+  apr_array_header_t *temp_props;
+  char *new_prop;
+  int i = 0;
+  int j = 0;
+
+  temp_props = apr_array_make(pool, 4, sizeof(char *));
+  new_prop = apr_palloc(pool, strlen(property)+1);
+
+  for (i = 0; property[i] != '\0'; i++)
+    {
+      if (property[i] != ';')
+        {
+          new_prop[j] = property[i];
+          j++;
+        }
+      else if (property[i] == ';')
+        {
+          if (property[i+1] == ';')
+            {
+              new_prop[j] = ';';
+              j++;
+              i++;
+            }
+          else
+            {
+              new_prop[j] = '\0';
+              APR_ARRAY_PUSH(temp_props, char *) = new_prop;
+              new_prop += j + 1;
+              j = 0;
+            }
+        }
+    }
+  new_prop[j] = '\0';
+  APR_ARRAY_PUSH(temp_props, char *) = new_prop;
+  *props = temp_props;
+}
+
 /* For one auto-props config entry (NAME, VALUE), if the filename pattern
    NAME matches BATON->filename case insensitively then add the properties
    listed in VALUE into BATON->properties.
@@ -93,9 +137,9 @@ auto_props_enumerator(const char *name,
                       void *baton,
                       apr_pool_t *pool)
 {
+  int i;
   auto_props_baton_t *autoprops = baton;
-  char *property;
-  char *last_token;
+  apr_array_header_t *props;
 
   /* nothing to do here without a value */
   if (strlen(value) == 0)
@@ -105,14 +149,13 @@ auto_props_enumerator(const char *name,
   if (apr_fnmatch(name, autoprops->filename, APR_FNM_CASE_BLIND) == APR_FNM_NOMATCH)
     return TRUE;
 
-  /* parse the value (we dup it first to effectively lose the
-     'const', and to avoid messing up the original value) */
-  property = apr_pstrdup(autoprops->pool, value);
-  property = apr_strtok(property, ";", &last_token);
-  while (property)
+  split_props(&props, value, autoprops->pool);
+
+  for (i = 0; i < props->nelts; i++)
     {
       int len;
       const char *this_value;
+      char *property = APR_ARRAY_IDX(props, i, char *);
       char *equal_sign = strchr(property, '=');
 
       if (equal_sign)
@@ -128,10 +171,13 @@ auto_props_enumerator(const char *name,
         }
       trim_string(&property);
       len = strlen(property);
+
       if (len > 0)
         {
-          svn_string_t *propval = svn_string_create(this_value,
-                                                    autoprops->pool);
+          svn_string_t *propval = apr_palloc(autoprops->pool,
+                                             sizeof(*propval));
+          propval->data = this_value;
+          propval->len = strlen(this_value);
 
           apr_hash_set(autoprops->properties, property, len, propval);
           if (strcmp(property, SVN_PROP_MIME_TYPE) == 0)
@@ -139,7 +185,6 @@ auto_props_enumerator(const char *name,
           else if (strcmp(property, SVN_PROP_EXECUTABLE) == 0)
             autoprops->have_executable = TRUE;
         }
-      property = apr_strtok(NULL, ";", &last_token);
     }
   return TRUE;
 }
