@@ -1,7 +1,7 @@
 /* revs-txns.c : operations on revision and transactions
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -15,7 +15,6 @@
  * ====================================================================
  */
 
-#include <assert.h>
 #include <string.h>
 
 #include <apr_tables.h>
@@ -25,6 +24,8 @@
 #include "svn_time.h"
 #include "svn_fs.h"
 #include "svn_props.h"
+#include "svn_hash.h"
+#include "svn_io.h"
 
 #include "fs.h"
 #include "dag.h"
@@ -41,6 +42,7 @@
 #include "../libsvn_fs/fs-loader.h"
 
 #include "svn_private_config.h"
+#include "private/svn_fs_util.h"
 
 
 /*** Helpers ***/
@@ -157,7 +159,7 @@ svn_fs_base__youngest_rev(svn_revnum_t *youngest_p,
                           apr_pool_t *pool)
 {
   svn_revnum_t youngest;
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_youngest_rev, &youngest,
                                  pool));
   *youngest_p = youngest;
@@ -192,7 +194,7 @@ svn_fs_base__revision_proplist(apr_hash_t **table_p,
   struct revision_proplist_args args;
   apr_hash_t *table;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.table_p = &table;
   args.rev = rev;
@@ -214,7 +216,7 @@ svn_fs_base__revision_prop(svn_string_t **value_p,
   struct revision_proplist_args args;
   apr_hash_t *table;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   /* Get the proplist. */
   args.table_p = &table;
@@ -272,11 +274,9 @@ txn_body_change_rev_prop(void *baton, trail_t *trail)
 {
   struct change_rev_prop_args *args = baton;
 
-  SVN_ERR(svn_fs_base__set_rev_prop(trail->fs, args->rev,
-                                    args->name, args->value, 
-                                    trail, trail->pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__set_rev_prop(trail->fs, args->rev,
+                                   args->name, args->value,
+                                   trail, trail->pool);
 }
 
 
@@ -289,14 +289,12 @@ svn_fs_base__change_rev_prop(svn_fs_t *fs,
 {
   struct change_rev_prop_args args;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.rev = rev;
   args.name = name;
   args.value = value;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_change_rev_prop, &args, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__retry_txn(fs, txn_body_change_rev_prop, &args, pool);
 }
 
 
@@ -312,8 +310,7 @@ svn_fs_base__txn_make_committed(svn_fs_t *fs,
 {
   transaction_t *txn;
 
-  /* Don't you dare call this with an invalid REVISION. */
-  assert(SVN_IS_VALID_REVNUM(revision));
+  SVN_ERR_ASSERT(SVN_IS_VALID_REVNUM(revision));
 
   /* Make sure the TXN is not committed already. */
   SVN_ERR(get_txn(&txn, fs, txn_name, FALSE, trail, pool));
@@ -425,7 +422,7 @@ svn_fs_base__add_txn_copy(svn_fs_t *fs,
     txn->copies = apr_array_make(pool, 1, sizeof(copy_id));
 
   /* Add COPY_ID to the array. */
-  (*((const char **)(apr_array_push(txn->copies)))) = copy_id;
+  APR_ARRAY_PUSH(txn->copies, const char *) = copy_id;
 
   /* Finally, write out the transaction. */
   return put_txn(fs, txn, txn_name, trail, pool);
@@ -484,7 +481,7 @@ svn_fs_base__txn_proplist(apr_hash_t **table_p,
   apr_hash_t *table;
   svn_fs_t *fs = txn->fs;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.table_p = &table;
   args.id = txn->id;
@@ -505,7 +502,7 @@ svn_fs_base__txn_prop(svn_string_t **value_p,
   apr_hash_t *table;
   svn_fs_t *fs = txn->fs;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   /* Get the proplist. */
   args.table_p = &table;
@@ -577,27 +574,48 @@ svn_fs_base__change_txn_prop(svn_fs_txn_t *txn,
   struct change_txn_prop_args args;
   svn_fs_t *fs = txn->fs;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.id = txn->id;
   args.name = name;
   args.value = value;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_change_txn_prop, &args, pool));
+  return svn_fs_base__retry_txn(fs, txn_body_change_txn_prop, &args, pool);
+}
+
+
+svn_error_t *
+svn_fs_base__change_txn_props(svn_fs_txn_t *txn,
+                              apr_array_header_t *props,
+                              apr_pool_t *pool)
+{
+  apr_pool_t *iterpool = svn_pool_create(pool);
+  int i;
+
+  for (i = 0; i < props->nelts; i++)
+    {
+      svn_prop_t *prop = &APR_ARRAY_IDX(props, i, svn_prop_t);
+
+      svn_pool_clear(iterpool);
+
+      SVN_ERR(svn_fs_base__change_txn_prop(txn, prop->name,
+                                           prop->value, iterpool));
+    }
+  svn_pool_destroy(iterpool);
 
   return SVN_NO_ERROR;
 }
 
-
 
 /* Creating a transaction */
 
-txn_vtable_t txn_vtable = {
+static txn_vtable_t txn_vtable = {
   svn_fs_base__commit_txn,
   svn_fs_base__abort_txn,
   svn_fs_base__txn_prop,
   svn_fs_base__txn_proplist,
   svn_fs_base__change_txn_prop,
-  svn_fs_base__txn_root
+  svn_fs_base__txn_root,
+  svn_fs_base__change_txn_props
 };
 
 
@@ -636,9 +654,9 @@ txn_body_begin_txn(void *baton, trail_t *trail)
   const svn_fs_id_t *root_id;
   const char *txn_id;
 
-  SVN_ERR(svn_fs_base__rev_get_root(&root_id, trail->fs, args->rev, 
+  SVN_ERR(svn_fs_base__rev_get_root(&root_id, trail->fs, args->rev,
                                     trail, trail->pool));
-  SVN_ERR(svn_fs_bdb__create_txn(&txn_id, trail->fs, root_id, 
+  SVN_ERR(svn_fs_bdb__create_txn(&txn_id, trail->fs, root_id,
                                  trail, trail->pool));
 
   if (args->flags & SVN_FS_TXN_CHECK_OOD)
@@ -646,7 +664,7 @@ txn_body_begin_txn(void *baton, trail_t *trail)
       struct change_txn_prop_args cpargs;
       cpargs.fs = trail->fs;
       cpargs.id = txn_id;
-      cpargs.name = SVN_FS_PROP_TXN_CHECK_OOD;
+      cpargs.name = SVN_FS__PROP_TXN_CHECK_OOD;
       cpargs.value = svn_string_create("true", trail->pool);
 
       SVN_ERR(txn_body_change_txn_prop(&cpargs, trail));
@@ -657,12 +675,12 @@ txn_body_begin_txn(void *baton, trail_t *trail)
       struct change_txn_prop_args cpargs;
       cpargs.fs = trail->fs;
       cpargs.id = txn_id;
-      cpargs.name = SVN_FS_PROP_TXN_CHECK_LOCKS;
+      cpargs.name = SVN_FS__PROP_TXN_CHECK_LOCKS;
       cpargs.value = svn_string_create("true", trail->pool);
 
       SVN_ERR(txn_body_change_txn_prop(&cpargs, trail));
     }
-    
+
   *args->txn_p = make_txn(trail->fs, txn_id, args->rev, trail->pool);
   return SVN_NO_ERROR;
 }
@@ -683,7 +701,7 @@ svn_fs_base__begin_txn(svn_fs_txn_t **txn_p,
   struct begin_txn_args args;
   svn_string_t date;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.txn_p = &txn;
   args.rev   = rev;
@@ -699,10 +717,8 @@ svn_fs_base__begin_txn(svn_fs_txn_t **txn_p,
      automatically overwritten with a revision datestamp. */
   date.data = svn_time_to_cstring(apr_time_now(), pool);
   date.len = strlen(date.data);
-  SVN_ERR(svn_fs_base__change_txn_prop(txn, SVN_PROP_REVISION_DATE,
-                                       &date, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__change_txn_prop(txn, SVN_PROP_REVISION_DATE,
+                                       &date, pool);
 }
 
 
@@ -728,7 +744,7 @@ txn_body_open_txn(void *baton, trail_t *trail)
       SVN_ERR(svn_fs_base__txn_get_revision(&base_rev, trail->fs, txn_id,
                                             trail, trail->pool));
     }
-  
+
   *args->txn_p = make_txn(trail->fs, args->name, base_rev, trail->pool);
   return SVN_NO_ERROR;
 }
@@ -743,7 +759,7 @@ svn_fs_base__open_txn(svn_fs_txn_t **txn_p,
   svn_fs_txn_t *txn;
   struct open_txn_args args;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.txn_p = &txn;
   args.name = name;
@@ -765,7 +781,7 @@ static svn_error_t *
 txn_body_cleanup_txn(void *baton, trail_t *trail)
 {
   struct cleanup_txn_args *args = baton;
-  return get_txn(args->txn_p, trail->fs, args->name, TRUE, 
+  return get_txn(args->txn_p, trail->fs, args->name, TRUE,
                  trail, trail->pool);
 }
 
@@ -773,7 +789,7 @@ txn_body_cleanup_txn(void *baton, trail_t *trail)
 static svn_error_t *
 txn_body_cleanup_txn_copy(void *baton, trail_t *trail)
 {
-  svn_error_t *err = svn_fs_bdb__delete_copy(trail->fs, baton, trail, 
+  svn_error_t *err = svn_fs_bdb__delete_copy(trail->fs, baton, trail,
                                              trail->pool);
 
   /* Copy doesn't exist?  No sweat. */
@@ -808,7 +824,7 @@ txn_body_get_dirents(void *baton, trail_t *trail)
   dag_node_t *node;
 
   /* Get the node. */
-  SVN_ERR(svn_fs_base__dag_get_node(&node, trail->fs, args->id, 
+  SVN_ERR(svn_fs_base__dag_get_node(&node, trail->fs, args->id,
                                     trail, trail->pool));
 
   /* If immutable, do nothing and return. */
@@ -821,7 +837,7 @@ txn_body_get_dirents(void *baton, trail_t *trail)
     return SVN_NO_ERROR;
 
   /* Else it's mutable.  Get its dirents. */
-  return svn_fs_base__dag_dir_entries(args->dirents, node, 
+  return svn_fs_base__dag_dir_entries(args->dirents, node,
                                       trail, trail->pool);
 }
 
@@ -914,7 +930,7 @@ svn_fs_base__purge_txn(svn_fs_t *fs,
   transaction_t *txn;
   int i;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   /* Open the transaction, expecting it to be dead. */
   args.txn_p = &txn;
@@ -971,7 +987,7 @@ svn_error_t *
 svn_fs_base__abort_txn(svn_fs_txn_t *txn,
                        apr_pool_t *pool)
 {
-  SVN_ERR(svn_fs_base__check_fs(txn->fs));
+  SVN_ERR(svn_fs__check_fs(txn->fs, TRUE));
 
   /* Set the transaction to "dead". */
   SVN_ERR(svn_fs_base__retry_txn(txn->fs, txn_body_abort_txn, txn, pool));
@@ -994,7 +1010,7 @@ static svn_error_t *
 txn_body_list_transactions(void* baton, trail_t *trail)
 {
   struct list_transactions_args *args = baton;
-  return svn_fs_bdb__get_txn_list(args->names_p, trail->fs, 
+  return svn_fs_bdb__get_txn_list(args->names_p, trail->fs,
                                   trail, args->pool);
 }
 
@@ -1006,7 +1022,7 @@ svn_fs_base__list_transactions(apr_array_header_t **names_p,
   apr_array_header_t *names;
   struct list_transactions_args args;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   args.names_p = &names;
   args.pool = pool;
