@@ -29,6 +29,7 @@
 #include "svn_mergeinfo.h"
 #include "private/svn_mergeinfo_private.h"
 #include "svn_private_config.h"
+#include "svn_hash.h"
 
 /* Attempt to combine two adjacent or overlapping ranges, IN1 and IN2, and put
    the result in OUTPUT.  Return whether they could be combined.
@@ -52,13 +53,11 @@ combine_ranges(svn_merge_range_t **output, svn_merge_range_t *in1,
     {
       if (!consider_inheritance
           || (consider_inheritance
-              && ((in1->inheritable ? TRUE : FALSE)
-                   == (in2->inheritable ? TRUE : FALSE))))
+              && (in1->inheritable == in2->inheritable)))
         {
           (*output)->start = MIN(in1->start, in2->start);
           (*output)->end = MAX(in1->end, in2->end);
-          (*output)->inheritable =
-            (in1->inheritable || in2->inheritable) ? TRUE : FALSE;
+          (*output)->inheritable = (in1->inheritable || in2->inheritable);
           return TRUE;
         }
     }
@@ -82,17 +81,15 @@ parse_pathname(const char **input, const char *end,
         last_colon = curr;
       curr++;
     }
-  
+
   if (!last_colon)
     return svn_error_create(SVN_ERR_MERGEINFO_PARSE_ERROR, NULL,
                             _("Pathname not terminated by ':'"));
-
-  *pathname = svn_stringbuf_create("", pool);
-  svn_stringbuf_appendbytes(*pathname, *input, (last_colon - *input));
-
-  if ((*pathname)->len == 0)
+  if (last_colon == *input)
     return svn_error_create(SVN_ERR_MERGEINFO_PARSE_ERROR, NULL,
-                            _("No pathname preceeding ':'"));
+                            _("No pathname preceding ':'"));
+
+  *pathname = svn_stringbuf_ncreate(*input, last_colon - *input, pool);
   *input = last_colon;
 
   return SVN_NO_ERROR;
@@ -116,7 +113,7 @@ parse_pathname(const char **input, const char *end,
 
      e.g.  *LASTRANGE: '4-10*' merged with MRANGE: '6'________
                   |                           |               |
-             Update end field               Push       Account for trimmed 
+             Update end field               Push       Account for trimmed
                   |                           |        range from *LASTRANGE.
                   |                           |        Push it last to
                   |                           |        maintain sort order.
@@ -146,7 +143,7 @@ combine_with_lastrange(svn_merge_range_t** lastrange,
   svn_merge_range_t *pushed_mrange_2 = NULL;
   svn_boolean_t ranges_intersect = FALSE;
   svn_boolean_t ranges_have_same_inheritance = FALSE;
-  
+
   if (*lastrange)
     {
       if ((*lastrange)->start <= mrange->end
@@ -168,7 +165,7 @@ combine_with_lastrange(svn_merge_range_t** lastrange,
          LASTRANGE and MRANGE "intersect" but have different
          inheritability and we are considering inheritance so
          can't combined them...
-         
+
          ...In all these cases just push MRANGE onto *LASTRANGE. */
       if (dup_mrange)
         pushed_mrange_1 = svn_merge_range_dup(mrange, pool);
@@ -184,7 +181,7 @@ combine_with_lastrange(svn_merge_range_t** lastrange,
           (*lastrange)->start = MIN((*lastrange)->start, mrange->start);
           (*lastrange)->end = MAX((*lastrange)->end, mrange->end);
           (*lastrange)->inheritable =
-            ((*lastrange)->inheritable || mrange->inheritable) ? TRUE : FALSE;
+            ((*lastrange)->inheritable || mrange->inheritable);
         }
       else /* Ranges intersect but have different
               inheritability so merge the ranges. */
@@ -212,7 +209,7 @@ combine_with_lastrange(svn_merge_range_t** lastrange,
                       pushed_mrange_1->start = pushed_mrange_1->start;
                       pushed_mrange_1->end = tmp_revnum;
                       *lastrange = pushed_mrange_1;
-                    } 
+                    }
                 }
               else /* (*lastrange)->end < mrange->end) */
                 {
@@ -295,6 +292,9 @@ combine_with_lastrange(svn_merge_range_t** lastrange,
                 }
               else /* ((*lastrange)->start > mrange->start) */
                 {
+                  pushed_mrange_2 =
+                    apr_palloc(pool, sizeof(*pushed_mrange_2));
+
                   if ((*lastrange)->end < mrange->end)
                     {
                       pushed_mrange_2->start = (*lastrange)->end;
@@ -351,10 +351,18 @@ range_to_string(svn_string_t **result, svn_merge_range_t *range,
     *result = svn_string_createf(pool, "%ld%s", range->end,
                                  range->inheritable
                                  ? "" : SVN_MERGEINFO_NONINHERITABLE_STR);
-  else
+  else if (range->start - 1 == range->end)
+    *result = svn_string_createf(pool, "-%ld%s", range->start,
+                                 range->inheritable
+                                 ? "" : SVN_MERGEINFO_NONINHERITABLE_STR);
+  else if (range->start < range->end)
     *result = svn_string_createf(pool, "%ld-%ld%s", range->start + 1,
                                  range->end, range->inheritable
                                  ? "" : SVN_MERGEINFO_NONINHERITABLE_STR);
+  else
+    *result = svn_string_createf(pool, "%ld-%ld%s", range->start,
+                                 range->end + 1, range->inheritable
+                                 ? "" : SVN_MERGEINFO_NONINHERITABLE_STR);
   return SVN_NO_ERROR;
 }
 
@@ -536,7 +544,7 @@ parse_revision_line(const char **input, const char *end, svn_mergeinfo_t hash,
                 }
 
               /* Combine overlapping or adjacent ranges with the
-                 same inheritability. */              
+                 same inheritability. */
               if (lastrange->inheritable == range->inheritable)
                 {
                   lastrange->end = MAX(range->end, lastrange->end);
@@ -567,7 +575,6 @@ parse_top(const char **input, const char *end, svn_mergeinfo_t hash,
   return SVN_NO_ERROR;
 }
 
-/* Parse mergeinfo.  */
 svn_error_t *
 svn_mergeinfo_parse(svn_mergeinfo_t *mergeinfo,
                     const char *input,
@@ -587,8 +594,6 @@ svn_mergeinfo_parse(svn_mergeinfo_t *mergeinfo,
 }
 
 
-/* Merge revision list RANGELIST into *MERGEINFO, doing some trivial
-   attempts to combine ranges as we go. */
 svn_error_t *
 svn_rangelist_merge(apr_array_header_t **rangelist,
                     apr_array_header_t *changes,
@@ -637,7 +642,7 @@ svn_rangelist_merge(apr_array_header_t **rangelist,
   /* Copy back any remaining elements.
      Only one of these loops should end up running, if anything. */
 
-  assert (!(i < (*rangelist)->nelts && j < changes->nelts));
+  SVN_ERR_ASSERT(!(i < (*rangelist)->nelts && j < changes->nelts));
 
   for (; i < (*rangelist)->nelts; i++)
     {
@@ -873,7 +878,6 @@ rangelist_intersect_or_remove(apr_array_header_t **output,
 }
 
 
-/* Expected to handle all the range overlap cases: non, partial, full */
 svn_error_t *
 svn_rangelist_intersect(apr_array_header_t **output,
                         apr_array_header_t *rangelist1,
@@ -896,116 +900,121 @@ svn_rangelist_remove(apr_array_header_t **output,
                                        consider_inheritance, pool);
 }
 
-/* Output deltas via *DELETED and *ADDED, which will never be @c NULL.
-
-   The following diagrams illustrate some common range delta scenarios:
-
-    (from)           deleted
-    r0 <===========(=========)============[=========]===========> rHEAD
-    [to]                                    added
-
-    (from)           deleted                deleted
-    r0 <===========(=========[============]=========)===========> rHEAD
-    [to]
-
-    (from)           deleted
-    r0 <===========(=========[============)=========]===========> rHEAD
-    [to]                                    added
-
-    (from)                                  deleted
-    r0 <===========[=========(============]=========)===========> rHEAD
-    [to]             added
-
-    (from)
-    r0 <===========[=========(============)=========]===========> rHEAD
-    [to]             added                  added
-
-    (from)  d                                  d             d
-    r0 <===(=[=)=]=[==]=[=(=)=]=[=]=[=(===|===(=)==|=|==[=(=]=)=> rHEAD
-    [to]        a   a    a   a   a   a                   a
-*/
 svn_error_t *
 svn_rangelist_diff(apr_array_header_t **deleted, apr_array_header_t **added,
                    apr_array_header_t *from, apr_array_header_t *to,
                    svn_boolean_t consider_inheritance,
                    apr_pool_t *pool)
 {
+  /* The following diagrams illustrate some common range delta scenarios:
+
+     (from)           deleted
+     r0 <===========(=========)============[=========]===========> rHEAD
+     [to]                                    added
+
+     (from)           deleted                deleted
+     r0 <===========(=========[============]=========)===========> rHEAD
+     [to]
+
+     (from)           deleted
+     r0 <===========(=========[============)=========]===========> rHEAD
+     [to]                                    added
+
+     (from)                                  deleted
+     r0 <===========[=========(============]=========)===========> rHEAD
+     [to]             added
+
+     (from)
+     r0 <===========[=========(============)=========]===========> rHEAD
+     [to]             added                  added
+
+     (from)  d                                  d             d
+     r0 <===(=[=)=]=[==]=[=(=)=]=[=]=[=(===|===(=)==|=|==[=(=]=)=> rHEAD
+     [to]        a   a    a   a   a   a                   a
+  */
+
   /* The items that are present in from, but not in to, must have been
      deleted. */
   SVN_ERR(svn_rangelist_remove(deleted, to, from, consider_inheritance,
                                pool));
   /* The items that are present in to, but not in from, must have been
      added.  */
-  SVN_ERR(svn_rangelist_remove(added, from, to, consider_inheritance, pool));
+  return svn_rangelist_remove(added, from, to, consider_inheritance, pool);
+}
+
+struct mergeinfo_diff_baton
+{
+  svn_mergeinfo_t from;
+  svn_mergeinfo_t to;
+  svn_mergeinfo_t deleted;
+  svn_mergeinfo_t added;
+  svn_boolean_t consider_inheritance;
+  apr_pool_t *pool;
+};
+
+/* This implements the 'svn_hash_diff_func_t' interface.
+   BATON is of type 'struct mergeinfo_diff_baton *'.
+*/
+static svn_error_t *
+mergeinfo_hash_diff_cb(const void *key, apr_ssize_t klen,
+                       enum svn_hash_diff_key_status status,
+                       void *baton)
+{
+  /* hash_a is FROM mergeinfo,
+     hash_b is TO mergeinfo. */
+  struct mergeinfo_diff_baton *cb = baton;
+  apr_array_header_t *from_rangelist, *to_rangelist;
+  const char *path = key;
+  if (status == svn_hash_diff_key_both)
+    {
+      /* Record any deltas (additions or deletions). */
+      apr_array_header_t *deleted_rangelist, *added_rangelist;
+      from_rangelist = apr_hash_get(cb->from, path, APR_HASH_KEY_STRING);
+      to_rangelist = apr_hash_get(cb->to, path, APR_HASH_KEY_STRING);
+      svn_rangelist_diff(&deleted_rangelist, &added_rangelist,
+                         from_rangelist, to_rangelist,
+                         cb->consider_inheritance, cb->pool);
+      if (cb->deleted && deleted_rangelist->nelts > 0)
+        apr_hash_set(cb->deleted, apr_pstrdup(cb->pool, path),
+                     APR_HASH_KEY_STRING, deleted_rangelist);
+      if (cb->added && added_rangelist->nelts > 0)
+        apr_hash_set(cb->added, apr_pstrdup(cb->pool, path),
+                     APR_HASH_KEY_STRING, added_rangelist);
+    }
+  else if ((status == svn_hash_diff_key_a) && cb->deleted)
+    {
+      from_rangelist = apr_hash_get(cb->from, path, APR_HASH_KEY_STRING);
+      apr_hash_set(cb->deleted, apr_pstrdup(cb->pool, path),
+                   APR_HASH_KEY_STRING,
+                   svn_rangelist_dup(from_rangelist, cb->pool));
+    }
+  else if ((status == svn_hash_diff_key_b) && cb->added)
+    {
+      to_rangelist = apr_hash_get(cb->to, path, APR_HASH_KEY_STRING);
+      apr_hash_set(cb->added, apr_pstrdup(cb->pool, path), APR_HASH_KEY_STRING,
+                   svn_rangelist_dup(to_rangelist, cb->pool));
+    }
   return SVN_NO_ERROR;
 }
 
 /* Record deletions and additions of entire range lists (by path
    presence), and delegate to svn_rangelist_diff() for delta
    calculations on a specific path. */
-/* ### TODO: Merge implementation with
-   ### libsvn_subr/sorts.c:svn_prop_diffs().  Factor out a generic
-   ### hash diffing function for addition to APR's apr_hash.h API. */
 static svn_error_t *
 walk_mergeinfo_hash_for_diff(svn_mergeinfo_t from, svn_mergeinfo_t to,
                              svn_mergeinfo_t deleted, svn_mergeinfo_t added,
                              svn_boolean_t consider_inheritance,
                              apr_pool_t *pool)
 {
-  apr_hash_index_t *hi;
-  const void *key;
-  void *val;
-  const char *path;
-  apr_array_header_t *from_rangelist, *to_rangelist;
+  struct mergeinfo_diff_baton mdb;
+  mdb.from = from;
+  mdb.to = to;
+  mdb.deleted = deleted;
+  mdb.added = added;
+  mdb.consider_inheritance = consider_inheritance;
+  mdb.pool = pool;
 
-  /* Handle path deletions and differences. */
-  for (hi = apr_hash_first(pool, from); hi; hi = apr_hash_next(hi))
-    {
-      apr_hash_this(hi, &key, NULL, &val);
-      path = key;
-      from_rangelist = val;
-
-      /* If the path is not present at all in the "to" hash, the
-         entire "from" rangelist is a deletion.  Paths which are
-         present in the "to" hash require closer scrutiny. */
-      to_rangelist = apr_hash_get(to, path, APR_HASH_KEY_STRING);
-      if (to_rangelist)
-        {
-          /* Record any deltas (additions or deletions). */
-          apr_array_header_t *deleted_rangelist, *added_rangelist;
-          svn_rangelist_diff(&deleted_rangelist, &added_rangelist,
-                             from_rangelist, to_rangelist,
-                             consider_inheritance, pool);
-          if (deleted && deleted_rangelist->nelts > 0)
-            apr_hash_set(deleted, apr_pstrdup(pool, path),
-                         APR_HASH_KEY_STRING, deleted_rangelist);
-          if (added && added_rangelist->nelts > 0)
-            apr_hash_set(added, apr_pstrdup(pool, path),
-                         APR_HASH_KEY_STRING, added_rangelist);
-        }
-      else if (deleted)
-        apr_hash_set(deleted, apr_pstrdup(pool, path), APR_HASH_KEY_STRING,
-                     svn_rangelist_dup(from_rangelist, pool));
-    }
-
-  /* Handle path additions. */
-  if (!added)
-    return SVN_NO_ERROR;
-
-  for (hi = apr_hash_first(pool, to); hi; hi = apr_hash_next(hi))
-    {
-      apr_hash_this(hi, &key, NULL, &val);
-      path = key;
-      to_rangelist = val;
-
-      /* If the path is not present in the "from" hash, the entire
-         "to" rangelist is an addition. */
-      if (apr_hash_get(from, path, APR_HASH_KEY_STRING) == NULL)
-        apr_hash_set(added, apr_pstrdup(pool, path), APR_HASH_KEY_STRING,
-                     svn_rangelist_dup(to_rangelist, pool));
-    }
-
-  return SVN_NO_ERROR;
+  return svn_hash_diff(from, to, mergeinfo_hash_diff_cb, &mdb, pool);
 }
 
 svn_error_t *
@@ -1158,9 +1167,8 @@ svn_mergeinfo_remove(svn_mergeinfo_t *mergeinfo, svn_mergeinfo_t eraser,
                      svn_mergeinfo_t whiteboard, apr_pool_t *pool)
 {
   *mergeinfo = apr_hash_make(pool);
-  SVN_ERR(walk_mergeinfo_hash_for_diff(whiteboard, eraser, *mergeinfo, NULL,
-                                       TRUE, pool));
-  return SVN_NO_ERROR;
+  return walk_mergeinfo_hash_for_diff(whiteboard, eraser, *mergeinfo, NULL,
+                                      TRUE, pool);
 }
 
 svn_error_t *
@@ -1223,7 +1231,7 @@ mergeinfo_to_stringbuf(svn_stringbuf_t **output,
           svn_stringbuf_appendcstr(*output,
                                    apr_psprintf(pool, "%s%s:%s",
                                                 prefix ? prefix : "",
-                                                (char *) elt.key,
+                                                (const char *) elt.key,
                                                 revlist->data));
           if (i < sorted->nelts - 1)
             svn_stringbuf_appendcstr(*output, "\n");
@@ -1250,8 +1258,7 @@ svn_mergeinfo_to_string(svn_string_t **output, svn_mergeinfo_t input,
   return SVN_NO_ERROR;
 }
 
-/* Perform an in-place sort of the rangelists in a mergeinfo hash.  */
-svn_error_t*
+svn_error_t *
 svn_mergeinfo_sort(svn_mergeinfo_t input, apr_pool_t *pool)
 {
   apr_hash_index_t *hi;
@@ -1269,8 +1276,8 @@ svn_mergeinfo_sort(svn_mergeinfo_t input, apr_pool_t *pool)
 }
 
 svn_mergeinfo_catalog_t
-svn_mergeinfo__catalog_dup(svn_mergeinfo_catalog_t mergeinfo_catalog,
-                           apr_pool_t *pool)
+svn_mergeinfo_catalog_dup(svn_mergeinfo_catalog_t mergeinfo_catalog,
+                          apr_pool_t *pool)
 {
   svn_mergeinfo_t new_mergeinfo_catalog = apr_hash_make(pool);
   apr_hash_index_t *hi;
@@ -1281,17 +1288,11 @@ svn_mergeinfo__catalog_dup(svn_mergeinfo_catalog_t mergeinfo_catalog,
     {
       const void *key;
       void *val;
-      const char *path;
-
-      svn_mergeinfo_t mergeinfo, mergeinfo_copy;
       apr_hash_this(hi, &key, NULL, &val);
-      path = key;
-      mergeinfo = val;
-      mergeinfo_copy = svn_mergeinfo_dup(mergeinfo, pool);
       apr_hash_set(new_mergeinfo_catalog,
-                   apr_pstrdup(pool, path),
+                   apr_pstrdup(pool, key),
                    APR_HASH_KEY_STRING,
-                   mergeinfo_copy);
+                   svn_mergeinfo_dup(val, pool));
     }
 
   return new_mergeinfo_catalog;
@@ -1425,11 +1426,11 @@ svn_mergeinfo__remove_empty_rangelists(svn_mergeinfo_t mergeinfo,
           void *value;
           const char *path;
           apr_array_header_t *rangelist;
-          
+
           apr_hash_this(hi, &key, NULL, &value);
           path = key;
           rangelist = value;
-          
+
           if (rangelist->nelts == 0)
             {
               apr_hash_set(mergeinfo, path, APR_HASH_KEY_STRING, NULL);
@@ -1460,10 +1461,11 @@ svn_mergeinfo__remove_prefix_from_catalog(svn_mergeinfo_catalog_t *out_catalog,
 
       apr_hash_this(hi, &key, &klen, &value);
       original_path = key;
-      assert(klen >= prefix_len);
-      assert(strncmp(key, prefix, prefix_len) == 0);
+      SVN_ERR_ASSERT(klen >= prefix_len);
+      SVN_ERR_ASSERT(strncmp(key, prefix, prefix_len) == 0);
 
-      apr_hash_set(*out_catalog, original_path + prefix_len, klen-prefix_len, value);
+      apr_hash_set(*out_catalog, original_path + prefix_len,
+                   klen-prefix_len, value);
     }
 
   return SVN_NO_ERROR;
@@ -1578,7 +1580,6 @@ svn_mergeinfo__to_formatted_string(svn_string_t **output,
 
   if (mergeinfo && apr_hash_count(mergeinfo))
     {
-      output_buf = svn_stringbuf_create("", pool);
       SVN_ERR(mergeinfo_to_stringbuf(&output_buf, mergeinfo,
                                      prefix ? prefix : "", pool));
       svn_stringbuf_appendcstr(output_buf, "\n");
