@@ -3,7 +3,7 @@
  * commandline client.
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -17,11 +17,7 @@
  * ====================================================================
  */
 
-/* ==================================================================== */
-
-
-
-/*** Includes. ***/
+#include <apr_xlate.h>  /* for APR_LOCALE_CHARSET */
 
 #define APR_WANT_STDIO
 #define APR_WANT_STRFUNC
@@ -31,6 +27,7 @@
 #include "svn_client.h"
 #include "svn_types.h"
 #include "svn_pools.h"
+
 #include "cl.h"
 
 #include "svn_private_config.h"
@@ -63,12 +60,10 @@ svn_cl__accept_from_word(const char *word)
     return svn_cl__accept_base;
   if (strcmp(word, SVN_CL__ACCEPT_WORKING) == 0)
     return svn_cl__accept_working;
-#if 0 /* not yet implemented */
   if (strcmp(word, SVN_CL__ACCEPT_MINE_CONFLICT) == 0)
     return svn_cl__accept_mine_conflict;
   if (strcmp(word, SVN_CL__ACCEPT_THEIRS_CONFLICT) == 0)
     return svn_cl__accept_theirs_conflict;
-#endif /* 0 */
   if (strcmp(word, SVN_CL__ACCEPT_MINE_FULL) == 0)
     return svn_cl__accept_mine_full;
   if (strcmp(word, SVN_CL__ACCEPT_THEIRS_FULL) == 0)
@@ -82,9 +77,10 @@ svn_cl__accept_from_word(const char *word)
 }
 
 
+/* Print on stdout a diff between the 'base' and 'merged' files, if both of
+ * those are available, else between 'their' and 'my' files, of DESC. */
 static svn_error_t *
-show_diff(svn_boolean_t *performed_edit,
-          const svn_wc_conflict_description_t *desc,
+show_diff(const svn_wc_conflict_description_t *desc,
           apr_pool_t *pool)
 {
   const char *path1, *path2;
@@ -111,18 +107,57 @@ show_diff(svn_boolean_t *performed_edit,
   SVN_ERR(svn_stream_for_stdout(&output, pool));
   SVN_ERR(svn_diff_file_diff_2(&diff, path1, path2,
                                options, pool));
-  SVN_ERR(svn_diff_file_output_unified2(output, diff,
-                                        path1, path2,
-                                        NULL, NULL,
-                                        APR_LOCALE_CHARSET,
-                                        pool));
-
-  *performed_edit = TRUE;
-
-  return SVN_NO_ERROR;
+  return svn_diff_file_output_unified3(output, diff,
+                                       path1, path2,
+                                       NULL, NULL,
+                                       APR_LOCALE_CHARSET,
+                                       NULL, FALSE,
+                                       pool);
 }
 
 
+/* Print on stdout just the conflict hunks of a diff among the 'base', 'their'
+ * and 'my' files of DESC. */
+static svn_error_t *
+show_conflicts(const svn_wc_conflict_description_t *desc,
+               apr_pool_t *pool)
+{
+  svn_diff_t *diff;
+  svn_stream_t *output;
+  svn_diff_file_options_t *options;
+
+  options = svn_diff_file_options_create(pool);
+  options->ignore_eol_style = TRUE;
+  SVN_ERR(svn_stream_for_stdout(&output, pool));
+  SVN_ERR(svn_diff_file_diff3_2(&diff,
+                                desc->base_file,
+                                desc->my_file,
+                                desc->their_file,
+                                options, pool));
+  /* ### Consider putting the markers/labels from
+     ### svn_wc__merge_internal in the conflict description. */
+  return svn_diff_file_output_merge2(output, diff,
+                                     desc->base_file,
+                                     desc->my_file,
+                                     desc->their_file,
+                                     _("||||||| ORIGINAL"),
+                                     _("<<<<<<< MINE (select with 'mc')"),
+                                     _(">>>>>>> THEIRS (select with 'tc')"),
+                                     "=======",
+                                     svn_diff_conflict_display_only_conflicts,
+                                     pool);
+}
+
+
+/* Run an external editor, passing it the 'merged' file in DESC, or, if the
+ * 'merged' file is null, return an error. The tool to use is determined by
+ * B->editor_cmd, B->config and environment variables; see
+ * svn_cl__edit_file_externally() for details.
+ *
+ * If the tool runs, set *PERFORMED_EDIT to true; if a tool is not
+ * configured or cannot run, do not touch *PERFORMED_EDIT, report the error
+ * on stderr, and return SVN_NO_ERROR; if any other error is encountered,
+ * return that error. */
 static svn_error_t *
 open_editor(svn_boolean_t *performed_edit,
             const svn_wc_conflict_description_t *desc,
@@ -163,6 +198,14 @@ open_editor(svn_boolean_t *performed_edit,
 }
 
 
+/* Run an external merge tool, passing it the 'base', 'their', 'my' and
+ * 'merged' files in DESC. The tool to use is determined by B->config and
+ * environment variables; see svn_cl__merge_file_externally() for details.
+ *
+ * If the tool runs, set *PERFORMED_EDIT to true; if a tool is not
+ * configured or cannot run, do not touch *PERFORMED_EDIT, report the error
+ * on stderr, and return SVN_NO_ERROR; if any other error is encountered,
+ * return that error.  */
 static svn_error_t *
 launch_resolver(svn_boolean_t *performed_edit,
                 const svn_wc_conflict_description_t *desc,
@@ -255,7 +298,7 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
             {
               SVN_ERR(svn_cmdline_fprintf(stderr, pool, "%s\n",
                                           err->message ? err->message :
-                                          _("No editor found,"
+                                          _("No editor found;"
                                             " leaving all conflicts.")));
               svn_error_clear(err);
               b->external_failed = TRUE;
@@ -264,7 +307,7 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
             {
               SVN_ERR(svn_cmdline_fprintf(stderr, pool, "%s\n",
                                           err->message ? err->message :
-                                          _("Error running editor,"
+                                          _("Error running editor;"
                                             " leaving all conflicts.")));
               svn_error_clear(err);
               b->external_failed = TRUE;
@@ -296,7 +339,7 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
             {
               SVN_ERR(svn_cmdline_fprintf(stderr, pool, "%s\n",
                                           err->message ? err->message :
-                                          _("No merge tool found,"
+                                          _("No merge tool found;"
                                             " leaving all conflicts.")));
               svn_error_clear(err);
               b->external_failed = TRUE;
@@ -305,7 +348,7 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
             {
               SVN_ERR(svn_cmdline_fprintf(stderr, pool, "%s\n",
                                           err->message ? err->message :
-                                          _("Error running merge tool"
+                                          _("Error running merge tool;"
                                             " leaving all conflicts.")));
               svn_error_clear(err);
               b->external_failed = TRUE;
@@ -337,7 +380,12 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
       const char *answer;
       char *prompt;
       svn_boolean_t diff_allowed = FALSE;
+      /* Have they done something that might have affected the merged
+         file (so that we need to save a .edited copy)? */
       svn_boolean_t performed_edit = FALSE;
+      /* Have they done *something* (edit, look at diff, etc) to
+         give them a rational basis for choosing (r)esolved? */
+      svn_boolean_t knows_something = FALSE;
 
       if (desc->kind == svn_wc_conflict_kind_text)
         SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
@@ -360,8 +408,8 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
 
               if (desc->my_file)
                 {
-                  SVN_ERR(svn_stringbuf_from_file(&myval, desc->my_file,
-                                                  subpool));
+                  SVN_ERR(svn_stringbuf_from_file2(&myval, desc->my_file,
+                                                   subpool));
                   SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
                         _("They want to delete the property, "
                           "you want to change the value to '%s'.\n"),
@@ -369,8 +417,8 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
                 }
               else
                 {
-                  SVN_ERR(svn_stringbuf_from_file(&theirval, desc->their_file,
-                                                  subpool));
+                  SVN_ERR(svn_stringbuf_from_file2(&theirval, desc->their_file,
+                                                   subpool));
                   SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
                         _("They want to change the property value to '%s', "
                           "you want to delete the property.\n"),
@@ -395,39 +443,68 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
           svn_pool_clear(subpool);
 
           prompt = apr_pstrdup(subpool, _("Select: (p) postpone"));
+
           if (diff_allowed)
-            prompt = apr_pstrcat(subpool, prompt,
-                                 _(", (df) diff-full, (e) edit"),
-                                 NULL);
+            {
+              prompt = apr_pstrcat(subpool, prompt,
+                                   _(", (df) diff-full, (e) edit"),
+                                   NULL);
+
+              if (knows_something)
+                prompt = apr_pstrcat(subpool, prompt, _(", (r) resolved"),
+                                     NULL);
+
+              if (! desc->is_binary &&
+                  desc->kind != svn_wc_conflict_kind_property)
+                prompt = apr_pstrcat(subpool, prompt,
+                                     _(",\n        (mc) mine-conflict, "
+                                       "(tc) theirs-conflict"),
+                                     NULL);
+            }
           else
-            prompt = apr_pstrcat(subpool, prompt,
-                                 _(", (mf) mine-full, (tf) theirs-full"),
-                                 NULL);
-          if (performed_edit)
-            prompt = apr_pstrcat(subpool, prompt, _(", (r) resolved"), NULL);
+            {
+              if (knows_something)
+                prompt = apr_pstrcat(subpool, prompt, _(", (r) resolved"),
+                                     NULL);
+              prompt = apr_pstrcat(subpool, prompt,
+                                   _(",\n        "
+                                     "(mf) mine-full, (tf) theirs-full"),
+                                   NULL);
+            }
 
           prompt = apr_pstrcat(subpool, prompt, ",\n        ", NULL);
           prompt = apr_pstrcat(subpool, prompt,
-                               _("(h) help for more options: "),
+                               _("(s) show all options: "),
                                NULL);
 
           SVN_ERR(svn_cmdline_prompt_user2(&answer, prompt, b->pb, subpool));
 
-          if (strcmp(answer, "h") == 0 || strcmp(answer, "?") == 0)
+          if (strcmp(answer, "s") == 0)
             {
               SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
-              _("  (p)  postpone    - mark the conflict to be "
-                "resolved later\n"
-                "  (df) diff-full   - show all changes made to merged file\n"
-                "  (e)  edit        - change merged file in an editor\n"
-                "  (r)  resolved    - accept merged version of file\n"
-                "  (mf) mine-full   - accept my version of entire file "
-                "(ignore their changes)\n"
-                "  (tf) theirs-full - accept their version of entire file "
-                "(lose my changes)\n"
-                "  (l)  launch      - launch external tool to "
-                "resolve conflict\n"
-                "  (h)  help        - show this list\n\n")));
+              _("\n"
+                "  (e)  edit             - change merged file in an editor\n"
+                "  (df) diff-full        - show all changes made to merged "
+                                          "file\n"
+                "  (r)  resolved         - accept merged version of file\n"
+                "\n"
+                "  (dc) display-conflict - show all conflicts "
+                                          "(ignoring merged version)\n"
+                "  (mc) mine-conflict    - accept my version for all "
+                                          "conflicts (same)\n"
+                "  (tc) theirs-conflict  - accept their version for all "
+                                          "conflicts (same)\n"
+                "\n"
+                "  (mf) mine-full        - accept my version of entire file "
+                                          "(even non-conflicts)\n"
+                "  (tf) theirs-full      - accept their version of entire "
+                                          "file (same)\n"
+                "\n"
+                "  (p)  postpone         - mark the conflict to be "
+                                          "resolved later\n"
+                "  (l)  launch           - launch external tool to "
+                                          "resolve conflict\n"
+                "  (s)  show all         - show this list\n\n")));
             }
           else if (strcmp(answer, "p") == 0)
             {
@@ -437,40 +514,92 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
             }
           else if (strcmp(answer, "mc") == 0)
             {
-              SVN_ERR(svn_cmdline_fprintf
-                      (stderr, subpool,
-                       _("Sorry, '(mc) mine for conflicts' "
-                         "is not yet implemented; see\n"
-        "http://subversion.tigris.org/issues/show_bug.cgi?id=3049\n\n")));
-              continue;
+              if (desc->is_binary)
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; cannot choose "
+                                                "based on conflicts in a "
+                                                "binary file.\n\n")));
+                  continue;
+                }
+              else if (desc->kind == svn_wc_conflict_kind_property)
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; cannot choose "
+                                                "based on conflicts for "
+                                                "properties.\n\n")));
+                  continue;
+                }
+
+              (*result)->choice = svn_wc_conflict_choose_mine_conflict;
+              if (performed_edit)
+                (*result)->save_merged = TRUE;
+              break;
             }
           else if (strcmp(answer, "tc") == 0)
             {
-              SVN_ERR(svn_cmdline_fprintf
-                      (stderr, subpool,
-                       _("Sorry, '(tc) theirs for conflicts' "
-                         "is not yet implemented; see\n"
-        "http://subversion.tigris.org/issues/show_bug.cgi?id=3049\n\n")));
-              continue;
+              if (desc->is_binary)
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; cannot choose "
+                                                "based on conflicts in a "
+                                                "binary file.\n\n")));
+                  continue;
+                }
+              else if (desc->kind == svn_wc_conflict_kind_property)
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; cannot choose "
+                                                "based on conflicts for "
+                                                "properties.\n\n")));
+                  continue;
+                }
+              (*result)->choice = svn_wc_conflict_choose_theirs_conflict;
+              if (performed_edit)
+                (*result)->save_merged = TRUE;
+              break;
             }
           else if (strcmp(answer, "mf") == 0)
             {
               (*result)->choice = svn_wc_conflict_choose_mine_full;
+              if (performed_edit)
+                (*result)->save_merged = TRUE;
               break;
             }
           else if (strcmp(answer, "tf") == 0)
             {
               (*result)->choice = svn_wc_conflict_choose_theirs_full;
+              if (performed_edit)
+                (*result)->save_merged = TRUE;
               break;
             }
           else if (strcmp(answer, "dc") == 0)
             {
-              SVN_ERR(svn_cmdline_fprintf
-                      (stderr, subpool,
-                       _("Sorry, '(dc) diff of conflicts' "
-                         "is not yet implemented; see\n"
-        "http://subversion.tigris.org/issues/show_bug.cgi?id=3048\n\n")));
-              continue;
+              if (desc->is_binary)
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; cannot "
+                                                "display conflicts for a "
+                                                "binary file.\n\n")));
+                  continue;
+                }
+              else if (desc->kind == svn_wc_conflict_kind_property)
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; cannot "
+                                                "display conflicts for "
+                                                "properties.\n\n")));
+                  continue;
+                }
+              else if (! (desc->my_file && desc->base_file && desc->their_file))
+                {
+                  SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
+                                              _("Invalid option; original "
+                                                "files not available.\n\n")));
+                  continue;
+                }
+              SVN_ERR(show_conflicts(desc, subpool));
+              knows_something = TRUE;
             }
           else if (strcmp(answer, "df") == 0)
             {
@@ -482,17 +611,24 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
                   continue;
                 }
 
-              SVN_ERR(show_diff(&performed_edit, desc, subpool));
+              SVN_ERR(show_diff(desc, subpool));
+              knows_something = TRUE;
             }
           else if (strcmp(answer, "e") == 0)
             {
               SVN_ERR(open_editor(&performed_edit, desc, b, subpool));
+              if (performed_edit)
+                knows_something = TRUE;
             }
           else if (strcmp(answer, "l") == 0)
             {
               if (desc->base_file && desc->their_file && desc->my_file
                     && desc->merged_file)
-                SVN_ERR(launch_resolver(&performed_edit, desc, b, subpool));
+                {
+                  SVN_ERR(launch_resolver(&performed_edit, desc, b, subpool));
+                  if (performed_edit)
+                    knows_something = TRUE;
+                }
               else
                 SVN_ERR(svn_cmdline_fprintf(stderr, subpool,
                                             _("Invalid option.\n\n")));
@@ -502,7 +638,7 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
               /* We only allow the user accept the merged version of
                  the file if they've edited it, or at least looked at
                  the diff. */
-              if (performed_edit)
+              if (knows_something)
                 {
                   (*result)->choice = svn_wc_conflict_choose_merged;
                   break;
@@ -559,7 +695,7 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
                 "(ignore upstream addition)\n"
                 "  (tf) theirs-full - accept incoming item "
                 "(overwrite pre-existing item)\n"
-                "  (h)  help        - show this list\n\n")));
+                "  (h)  help        - show this help\n\n")));
             }
           if (strcmp(answer, "p") == 0)
             {
@@ -576,24 +712,6 @@ svn_cl__conflict_handler(svn_wc_conflict_result_t **result,
               (*result)->choice = svn_wc_conflict_choose_theirs_full;
               break;
             }
-          if (strcmp(answer, "mc") == 0)
-            {
-              SVN_ERR(svn_cmdline_fprintf
-                      (stderr, subpool,
-                       _("Sorry, '(mc) mine for conflicts' "
-                         "is not yet implemented; see\n"
-        "http://subversion.tigris.org/issues/show_bug.cgi?id=3049\n\n")));
-              continue;
-            }
-          if (strcmp(answer, "tc") == 0)
-            {
-              SVN_ERR(svn_cmdline_fprintf
-                      (stderr, subpool,
-                       _("Sorry, '(tc) theirs for conflicts' "
-                         "is not yet implemented; see\n"
-        "http://subversion.tigris.org/issues/show_bug.cgi?id=3049\n\n")));
-              continue;
-            }
         }
     }
 
