@@ -238,12 +238,19 @@ svn_cl__merge_file_externally(const char *base_path,
            "configuration option were not set.\n"));
 
   {
-    const char *arguments[] = { merge_tool, base_path, their_path,
-                                my_path, merged_path, NULL};
+    const char *arguments[6] = { 0 };
     char *cwd;
     apr_status_t status = apr_filepath_get(&cwd, APR_FILEPATH_NATIVE, pool);
     if (status != 0)
       return svn_error_wrap_apr(status, NULL);
+
+    arguments[0] = merge_tool;
+    arguments[1] = base_path;
+    arguments[2] = their_path;
+    arguments[3] = my_path;
+    arguments[4] = merged_path;
+    arguments[5] = NULL;
+
     return svn_io_run_cmd(svn_path_internal_style(cwd, pool), merge_tool,
                           arguments, NULL, NULL, TRUE, NULL, NULL, NULL,
                           pool);
