@@ -1,7 +1,7 @@
 /* fs-test.c --- tests for the filesystem
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -19,13 +19,13 @@
 #include <string.h>
 #include <apr_pools.h>
 
+#include "../svn_test.h"
+
 #include "svn_pools.h"
 #include "svn_time.h"
 #include "svn_string.h"
 #include "svn_fs.h"
-#include "svn_md5.h"
 
-#include "../svn_test.h"
 #include "../svn_test_fs.h"
 
 #include "../../libsvn_fs_base/id.h"
@@ -33,6 +33,7 @@
 #include "../../libsvn_fs_base/bdb/txn-table.h"
 #include "../../libsvn_fs_base/bdb/nodes-table.h"
 
+#include "private/svn_fs_util.h"
 #include "../../libsvn_delta/delta.h"
 
 #define SET_STR(ps, s) ((ps)->data = (s), (ps)->len = strlen(s))
@@ -57,9 +58,9 @@ create_berkeley_filesystem(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create and close a repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-create-berkeley",
-                              "bdb", pool));
-  
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-create-berkeley", opts,
+                                  pool));
+
   return SVN_NO_ERROR;
 }
 
@@ -87,8 +88,8 @@ open_berkeley_filesystem(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create and close a repository (using fs). */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-open-berkeley",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-open-berkeley", opts,
+                                  pool));
 
   /* Create a different fs object, and use it to re-open the
      repository again.  */
@@ -128,7 +129,7 @@ check_entry(svn_fs_root_t *root,
 
 /* Return an error if entry NAME is absent in directory PATH under ROOT. */
 static svn_error_t *
-check_entry_present(svn_fs_root_t *root, const char *path, 
+check_entry_present(svn_fs_root_t *root, const char *path,
                     const char *name, apr_pool_t *pool)
 {
   svn_boolean_t present;
@@ -145,7 +146,7 @@ check_entry_present(svn_fs_root_t *root, const char *path,
 
 /* Return an error if entry NAME is present in directory PATH under ROOT. */
 static svn_error_t *
-check_entry_absent(svn_fs_root_t *root, const char *path, 
+check_entry_absent(svn_fs_root_t *root, const char *path,
                    const char *name, apr_pool_t *pool)
 {
   svn_boolean_t present;
@@ -175,7 +176,7 @@ txn_body_check_id(void *baton, trail_t *trail)
   node_revision_t *noderev;
   svn_error_t *err;
 
-  err = svn_fs_bdb__get_node_revision(&noderev, args->fs, args->id, 
+  err = svn_fs_bdb__get_node_revision(&noderev, args->fs, args->id,
                                       trail, trail->pool);
 
   if (err && (err->apr_err == SVN_ERR_FS_ID_NOT_FOUND))
@@ -277,8 +278,8 @@ abort_txn(const char **msg,
     return SVN_NO_ERROR;
 
   /* Prepare two txns to receive the Greek tree. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-abort-txn",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-abort-txn", opts,
+                                  pool));
   SVN_ERR(svn_fs_begin_txn(&txn1, fs, 0, pool));
   SVN_ERR(svn_fs_begin_txn(&txn2, fs, 0, pool));
   SVN_ERR(svn_fs_txn_root(&txn1_root, txn1, pool));
@@ -287,7 +288,7 @@ abort_txn(const char **msg,
   /* Save their names for later. */
   SVN_ERR(svn_fs_txn_name(&txn1_name, txn1, pool));
   SVN_ERR(svn_fs_txn_name(&txn2_name, txn2, pool));
-  
+
   /* Create greek trees in them. */
   SVN_ERR(svn_test__create_greek_tree(txn1_root, pool));
   SVN_ERR(svn_test__create_greek_tree(txn2_root, pool));
@@ -328,7 +329,7 @@ abort_txn(const char **msg,
       *t1_pi_id,      *t2_pi_id,
       *t1_rho_id,     *t2_rho_id,
       *t1_tau_id,     *t2_tau_id;
-    
+
     SVN_ERR(svn_fs_node_id(&t1_root_id, txn1_root, "", pool));
     SVN_ERR(svn_fs_node_id(&t2_root_id, txn2_root, "", pool));
     SVN_ERR(svn_fs_node_id(&t1_iota_id, txn1_root, "iota", pool));
@@ -376,7 +377,7 @@ abort_txn(const char **msg,
     SVN_ERR(svn_fs_abort_txn(txn2, pool));
 
     /* Now test that all the nodes in txn2 at the time of the abort
-     * are gone, but all of the ones in txn1 are still there. 
+     * are gone, but all of the ones in txn1 are still there.
      */
 
     /* Check that every node rev in t2 has vanished from the fs. */
@@ -401,7 +402,7 @@ abort_txn(const char **msg,
     SVN_ERR(check_id_absent(fs, t2_pi_id, pool));
     SVN_ERR(check_id_absent(fs, t2_rho_id, pool));
     SVN_ERR(check_id_absent(fs, t2_tau_id, pool));
-    
+
     /* Check that every node rev in t1 is still in the fs. */
     SVN_ERR(check_id_present(fs, t1_root_id, pool));
     SVN_ERR(check_id_present(fs, t1_iota_id, pool));
@@ -493,9 +494,9 @@ abort_txn(const char **msg,
 
 /* This tests deleting of mutable nodes.  We build a tree in a
  * transaction, then try to delete various items in the tree.  We
- * never commit the tree, so every entry being deleted points to a 
- * mutable node. 
- * 
+ * never commit the tree, so every entry being deleted points to a
+ * mutable node.
+ *
  * ### todo: this test was written before commits worked.  It might
  * now be worthwhile to combine it with delete().
  */
@@ -516,11 +517,11 @@ delete_mutables(const char **msg,
     return SVN_NO_ERROR;
 
   /* Prepare a txn to receive the greek tree. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-del-from-dir",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-del-from-dir", opts,
+                                  pool));
   SVN_ERR(svn_fs_begin_txn(&txn, fs, 0, pool));
   SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
-  
+
   /* Create the greek tree. */
   SVN_ERR(svn_test__create_greek_tree(txn_root, pool));
 
@@ -682,7 +683,7 @@ delete_mutables(const char **msg,
 
 
 /* This tests deleting in general.
- * 
+ *
  * ### todo: this test was written after (and independently of)
  * delete_mutables().  It might be worthwhile to combine them.
  */
@@ -712,8 +713,8 @@ delete(const char **msg,
    */
 
   /* Prepare a txn to receive the greek tree. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-del-tree",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-del-tree", opts,
+                                  pool));
   SVN_ERR(svn_fs_begin_txn(&txn, fs, 0, pool));
   SVN_ERR(svn_fs_txn_root(&txn_root, txn, pool));
 
@@ -962,7 +963,7 @@ delete(const char **msg,
         /* path, contents (0 = dir) */
         { "iota",        "This is the file 'iota'.\n" }
       };
-      
+
       SVN_ERR(svn_test__validate_tree(txn_root, expected_entries, 1, pool));
     }
   }
@@ -1125,9 +1126,9 @@ canonicalize_abspath(const char **msg,
                      svn_boolean_t msg_only,
                      svn_test_opts_t *opts,
                      apr_pool_t *pool)
-{ 
+{
   apr_size_t i;
-  const char *paths[21][2] = 
+  const char *paths[21][2] =
     /* in                      out */
   { { NULL,                    NULL },
     { "",                      "/" },
@@ -1152,7 +1153,7 @@ canonicalize_abspath(const char **msg,
     { "///foo///bar///baz///", "/foo/bar/baz" },
   };
 
-  *msg = "test svn_fs_base__canonicalize_abspath";
+  *msg = "test svn_fs__canonicalize_abspath";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -1161,8 +1162,8 @@ canonicalize_abspath(const char **msg,
     {
       const char *input = paths[i][0];
       const char *output = paths[i][1];
-      const char *actual = svn_fs_base__canonicalize_abspath(input, pool);
-      
+      const char *actual = svn_fs__canonicalize_abspath(input, pool);
+
       if ((! output) && (! actual))
         continue;
       if ((! output) && actual)
@@ -1185,22 +1186,22 @@ create_within_copy(const char **msg,
                    svn_boolean_t msg_only,
                    svn_test_opts_t *opts,
                    apr_pool_t *pool)
-{ 
+{
   apr_pool_t *spool = svn_pool_create(pool);
   svn_fs_t *fs;
   svn_fs_txn_t *txn;
   svn_fs_root_t *txn_root, *rev_root;
   int i;
   svn_revnum_t youngest_rev = 0;
-  
+
   *msg = "create new items within a copied directory";
 
   if (msg_only)
     return SVN_NO_ERROR;
 
   /* Create a filesystem and repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-create-within-copy",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-create-within-copy", opts,
+                                  pool));
 
   /*** Revision 1:  Create the greek tree in revision.  ***/
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, spool));
@@ -1313,14 +1314,14 @@ skip_deltas(const char **msg,
             svn_boolean_t msg_only,
             svn_test_opts_t *opts,
             apr_pool_t *pool)
-{ 
+{
   svn_fs_t *fs;
   svn_fs_txn_t *txn;
   svn_fs_root_t *txn_root, *rev_root;
   apr_pool_t *subpool = svn_pool_create(pool);
   svn_revnum_t youngest_rev = 0;
   const char *one_line = "This is a line in file 'f'.\n";
-  svn_stringbuf_t *f = svn_stringbuf_create(one_line, pool); 
+  svn_stringbuf_t *f = svn_stringbuf_create(one_line, pool);
 
   *msg = "test skip deltas";
 
@@ -1328,8 +1329,8 @@ skip_deltas(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a filesystem and repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-skip-deltas",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-skip-deltas", opts,
+                                  pool));
 
   /* Create the file. */
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
@@ -1344,7 +1345,7 @@ skip_deltas(const char **msg,
   while (youngest_rev <= 128)
     {
       /* Append another line to the ever-growing file contents. */
-      svn_stringbuf_appendcstr(f, one_line); 
+      svn_stringbuf_appendcstr(f, one_line);
 
       /* Commit the new contents. */
       SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
@@ -1360,8 +1361,8 @@ skip_deltas(const char **msg,
   SVN_ERR(svn_test__get_file_contents(rev_root, "f", &f, pool));
   if (strcmp(one_line, f->data) != 0)
     return svn_error_createf
-      (SVN_ERR_TEST_FAILED, NULL, 
-       "Wrong contents.  Expected:\n   '%s'\nGot:\n   '%s'\n", 
+      (SVN_ERR_TEST_FAILED, NULL,
+       "Wrong contents.  Expected:\n   '%s'\nGot:\n   '%s'\n",
        one_line, f->data);
 
   svn_pool_destroy(subpool);
@@ -1381,7 +1382,7 @@ static svn_error_t *
 txn_body_get_txn(void *baton, trail_t *trail)
 {
   struct get_txn_args *args = baton;
-  return svn_fs_bdb__get_txn(args->txn, args->fs, args->txn_name, 
+  return svn_fs_bdb__get_txn(args->txn, args->fs, args->txn_name,
                              trail, trail->pool);
 }
 
@@ -1391,7 +1392,7 @@ redundant_copy(const char **msg,
                svn_boolean_t msg_only,
                svn_test_opts_t *opts,
                apr_pool_t *pool)
-{ 
+{
   svn_fs_t *fs;
   svn_fs_txn_t *txn;
   const char *txn_name;
@@ -1400,15 +1401,15 @@ redundant_copy(const char **msg,
   const svn_fs_id_t *old_D_id, *new_D_id;
   svn_revnum_t youngest_rev = 0;
   struct get_txn_args args;
-  
+
   *msg = "ensure no-op for redundant copies";
 
   if (msg_only)
     return SVN_NO_ERROR;
 
   /* Create a filesystem and repository. */
-  SVN_ERR(svn_test__create_fs(&fs, "test-repo-redundant-copy",
-                              "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-redundant-copy", opts,
+                                  pool));
 
   /* Create the greek tree in revision 1. */
   SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, pool));
@@ -1454,7 +1455,7 @@ redundant_copy(const char **msg,
   SVN_ERR(svn_fs_node_id(&new_D_id, txn_root, "A/D", pool));
   if (! svn_string_compare(svn_fs_unparse_id(old_D_id, pool),
                            svn_fs_unparse_id(new_D_id, pool)))
-    return svn_error_create 
+    return svn_error_create
       (SVN_ERR_TEST_FAILED, NULL,
        "Expected equivalent node-rev-ids; got differing ones");
 
@@ -1462,6 +1463,72 @@ redundant_copy(const char **msg,
 }
 
 
+static svn_error_t *
+orphaned_textmod_change(const char **msg,
+                        svn_boolean_t msg_only,
+                        svn_test_opts_t *opts,
+                        apr_pool_t *pool)
+{
+  apr_pool_t *subpool = svn_pool_create(pool);
+  svn_fs_t *fs;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root, *root;
+  svn_revnum_t youngest_rev = 0;
+  svn_txdelta_window_handler_t wh_func;
+  void *wh_baton;
+  apr_hash_t *changed_paths;
+
+  *msg = "test for orphaned textmod changed paths";
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* Create a filesystem and repository. */
+  SVN_ERR(svn_test__create_bdb_fs(&fs, "test-repo-orphaned-changes", opts,
+                                  pool));
+
+  /* Revision 1:  Create and commit the greek tree. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, 0, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_test__create_greek_tree(txn_root, subpool));
+  SVN_ERR(svn_fs_commit_txn(NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear(subpool);
+
+  /* Revision 2:  Start to change "iota", but don't complete the work. */
+  SVN_ERR(svn_fs_begin_txn(&txn, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_txn_root(&txn_root, txn, subpool));
+  SVN_ERR(svn_fs_apply_textdelta
+          (&wh_func, &wh_baton, txn_root, "iota", NULL, NULL, subpool));
+
+  /* Don't send any delta windows, but do commit the transaction.
+     According to the FS API docs, this is not a legal codepath.  But
+     this requirement on the API was added *after* its BDB
+     implementation, and the BDB backend can't enforce compliance with
+     the additional API rules in this case.  So we are really just
+     testing that misbehaving callers don't introduce more damage to
+     the repository than they have to. */
+  SVN_ERR(svn_fs_commit_txn(NULL, &youngest_rev, txn, subpool));
+  svn_pool_clear(subpool);
+
+  /* Fetch changed paths for the youngest revision.  We should find none. */
+  SVN_ERR(svn_fs_revision_root(&root, fs, youngest_rev, subpool));
+  SVN_ERR(svn_fs_paths_changed(&changed_paths, root, subpool));
+  if (apr_hash_count(changed_paths) != 0)
+    {
+      svn_fs_path_change_t *change = apr_hash_get(changed_paths, "/iota",
+                                                  APR_HASH_KEY_STRING);
+      if (change && change->text_mod)
+        return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
+                                "Got unexpected textmods changed path "
+                                "for 'iota'");
+      else
+        return svn_error_create(SVN_ERR_TEST_FAILED, NULL,
+                                "Got non-empty changed paths hash where empty "
+                                "one expected");
+    }
+
+  return SVN_NO_ERROR;
+}
+
 /* ------------------------------------------------------------------------ */
 
 /* The test table.  */
@@ -1478,5 +1545,6 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS(canonicalize_abspath),
     SVN_TEST_PASS(skip_deltas),
     SVN_TEST_PASS(redundant_copy),
+    SVN_TEST_PASS(orphaned_textmod_change),
     SVN_TEST_NULL
   };
