@@ -23,10 +23,12 @@
 #include <apr.h>
 
 #include "svn_error.h"
+#include "private/svn_skel.h"
 
 #include "../svn_test.h"
 #include "../svn_test_fs.h"
-#include "../../libsvn_fs_base/util/skel.h"
+#include "../../libsvn_fs/fs-loader.h"
+#include "../../libsvn_fs_base/fs.h"
 #include "../../libsvn_fs_base/util/fs_skels.h"
 #include "../../libsvn_fs_base/bdb/strings-table.h"
 #include "../../libsvn_fs_base/bdb/reps-table.h"
@@ -39,7 +41,7 @@ struct rep_args
 {
   const char *key;
   svn_fs_t *fs;
-  skel_t *skel;
+  svn_skel_t *skel;
 };
 
 
@@ -70,9 +72,10 @@ txn_body_read_rep(void *baton, trail_t *trail)
 {
   struct rep_args *b = (struct rep_args *) baton;
   representation_t *rep;
+  base_fs_data_t *bfd = b->fs->fsap_data;
   SVN_ERR(svn_fs_bdb__read_rep(&rep, b->fs, b->key, trail, trail->pool));
   return svn_fs_base__unparse_representation_skel(&(b->skel), rep,
-                                                  trail->pool);
+                                                  bfd->format, trail->pool);
 }
 
 
@@ -103,13 +106,13 @@ write_new_rep(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-write-new-rep",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-write-new-rep", opts,
+           pool));
 
   /* Set up transaction baton */
   args.fs = fs;
-  args.skel = svn_fs_base__parse_skel(rep, strlen(rep), pool);
+  args.skel = svn_skel__parse(rep, strlen(rep), pool);
   args.key = NULL;
 
   /* Write new rep to reps table. */
@@ -142,13 +145,13 @@ write_rep(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-write-rep",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-write-rep", opts,
+           pool));
 
   /* Set up transaction baton */
   new_args.fs = fs;
-  new_args.skel = svn_fs_base__parse_skel(new_rep, strlen(new_rep), pool);
+  new_args.skel = svn_skel__parse(new_rep, strlen(new_rep), pool);
   new_args.key = NULL;
 
   /* Write new rep to reps table. */
@@ -162,7 +165,7 @@ write_rep(const char **msg,
 
   /* Set up transaction baton for re-writing reps. */
   args.fs = new_args.fs;
-  args.skel = svn_fs_base__parse_skel(rep, strlen(rep), pool);
+  args.skel = svn_skel__parse(rep, strlen(rep), pool);
   args.key = new_args.key;
 
   /* Overwrite first rep in reps table. */
@@ -220,14 +223,14 @@ read_rep(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-read-rep",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-read-rep", opts,
+           pool));
 
   /* Set up transaction baton */
   new_args.fs = fs;
-  new_args.skel = svn_fs_base__parse_skel(new_rep_before,
-                                          strlen(new_rep_before), pool);
+  new_args.skel = svn_skel__parse(new_rep_before, strlen(new_rep_before),
+                                  pool);
   new_args.key = NULL;
 
   /* Write new rep to reps table. */
@@ -251,14 +254,14 @@ read_rep(const char **msg,
     return svn_error_create(SVN_ERR_FS_GENERAL, NULL,
                             "error reading new representation");
 
-  skel_data = svn_fs_base__unparse_skel(read_args.skel, pool);
+  skel_data = svn_skel__unparse(read_args.skel, pool);
   if (memcmp(skel_data->data, new_rep_after, new_rep_after_len) != 0)
     return svn_error_createf(SVN_ERR_FS_GENERAL, NULL,
                              "representation corrupted (first check)");
 
   /* Set up transaction baton for re-writing reps. */
   args.fs = new_args.fs;
-  args.skel = svn_fs_base__parse_skel(rep, strlen(rep), pool);
+  args.skel = svn_skel__parse(rep, strlen(rep), pool);
   args.key = new_args.key;
 
   /* Overwrite first rep in reps table. */
@@ -277,7 +280,7 @@ read_rep(const char **msg,
     return svn_error_create(SVN_ERR_FS_GENERAL, NULL,
                             "error reading new representation");
 
-  skel_data = svn_fs_base__unparse_skel(read_args.skel, pool);
+  skel_data = svn_skel__unparse(read_args.skel, pool);
   if (memcmp(skel_data->data, rep_after, rep_after_len) != 0)
     return svn_error_createf(SVN_ERR_FS_GENERAL, NULL,
                              "representation corrupted (second check)");
@@ -305,13 +308,13 @@ delete_rep(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-delete-rep",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-delete-rep", opts,
+           pool));
 
   /* Set up transaction baton */
   new_args.fs = fs;
-  new_args.skel = svn_fs_base__parse_skel(new_rep, strlen(new_rep), pool);
+  new_args.skel = svn_skel__parse(new_rep, strlen(new_rep), pool);
   new_args.key = NULL;
 
   /* Write new rep to reps table. */
@@ -540,9 +543,9 @@ test_strings(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-test-strings",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-test-strings", opts,
+           pool));
 
   /* The plan (after each step below, verify the size and contents of
      the string):
@@ -645,9 +648,9 @@ write_null_string(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-test-strings",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-test-strings", opts,
+           pool));
 
   args.fs = fs;
   args.key = NULL;
@@ -675,9 +678,9 @@ abort_string(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-abort-string",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-abort-string", opts,
+           pool));
 
   /* The plan:
 
@@ -743,9 +746,9 @@ copy_string(const char **msg,
     return SVN_NO_ERROR;
 
   /* Create a new fs and repos */
-  SVN_ERR(svn_test__create_fs
-          (&fs, "test-repo-copy-string",
-           "bdb", pool));
+  SVN_ERR(svn_test__create_bdb_fs
+          (&fs, "test-repo-copy-string", opts,
+           pool));
 
   /*  Write a new string (string1). */
   args.fs = fs;
